//
// Created by robin on 1/8/18.
//

#include "dns-base/QDnsLabel.h"
#include "dns-base/QDnsQuery.h"

namespace QDns
{
    namespace Base
    {
        QDnsQuery::QDnsQuery()
                : m_isValid(true)
                  , m_type(RecordType::A)
                  , m_class(RecordClass::IN)
        {
        }

        QDnsQuery::QDnsQuery(const QByteArray &packetData, quint32 pos)
                : QDnsQuery()
        {
            QDataStream dataStream(packetData);
            dataStream.skipRawData(pos);
            parse(packetData, dataStream);
        }

        QDnsQuery::QDnsQuery(const QByteArray &packetData, QDataStream &dataStream)
                : QDnsQuery()
        {
            parse(packetData, dataStream);
        }

        void QDnsQuery::parse(const QByteArray &packetData, QDataStream &dataStream)
        {
            m_name = QDnsLabel::parse(packetData, dataStream);
            if (m_name.isNull() || dataStream.device()->bytesAvailable() < 4)
            {
                m_isValid = false;
            }
            else
            {
                dataStream >> m_type;
                dataStream >> m_class;
            }
        }

        QByteArray QDnsQuery::serialize() const
        {
            QByteArray byteArray;
            QDataStream dataStream(&byteArray, QIODevice::WriteOnly);
            serialize(dataStream);
            return byteArray;
        }

        void QDnsQuery::serialize(QDataStream &dataStream) const
        {
            QDnsLabel::serialize(m_name, dataStream);
            dataStream << m_type;
            dataStream << m_class;
        }

        bool QDnsQuery::isValid() const
        {
            return m_isValid;
        }

        const QString &QDnsQuery::getName() const
        {
            return m_name;
        }

        void QDnsQuery::setName(const QString &name)
        {
            m_name = name;
        }

        RecordType QDnsQuery::getType() const
        {
            return (RecordType) m_type;
        }

        void QDnsQuery::setType(RecordType type)
        {
            m_type = type;
        }

        RecordClass QDnsQuery::getClass() const
        {
            return (RecordClass) m_class;
        }

        void QDnsQuery::setClass(RecordClass aClass)
        {
            m_class = aClass;
        }
    }
}