//
// Created by robin on 1/23/18.
//

#include "dns-base/QDnsLabel.h"
#include "dns-base/RecordData/QDnsRecordDataPTR.h"

namespace QDns
{
    namespace Base
    {
        QDnsRecordDataPTR::QDnsRecordDataPTR()
                : QDnsRecordData()
        {
        }

        QDnsRecordDataPTR::QDnsRecordDataPTR(const QByteArray &packetData, quint32 pos)
                : QDnsRecordDataPTR()
        {
            QDataStream dataStream(packetData);
            dataStream.skipRawData(pos);
            parse(packetData, dataStream);
        }

        QDnsRecordDataPTR::QDnsRecordDataPTR(const QByteArray &packetData, QDataStream &dataStream)
                : QDnsRecordDataPTR()
        {
            parse(packetData, dataStream);
        }

        void QDnsRecordDataPTR::parse(const QByteArray &packetData, QDataStream &dataStream)
        {
            if (dataStream.device()->bytesAvailable() < 1)
            {
                m_isValid = false;
            }
            else
            {
                m_hostname = QDnsLabel::parse(packetData, dataStream);
                if (m_hostname.isNull())
                {
                    m_isValid = false;
                }
            }
        }

        QByteArray QDnsRecordDataPTR::serialize() const
        {
            QByteArray byteArray;
            QDataStream dataStream(&byteArray, QIODevice::WriteOnly);
            serialize(dataStream);
            return byteArray;
        }

        void QDnsRecordDataPTR::serialize(QDataStream &dataStream) const
        {
            QDnsLabel::serialize(m_hostname, dataStream);
        }

        QDnsRecordData *QDnsRecordDataPTR::clone() const
        {
            auto data = new QDnsRecordDataPTR();
            data->m_isValid = m_isValid;
            data->m_hostname = m_hostname;
            return data;
        }

        const QString &QDnsRecordDataPTR::getHostname() const
        {
            return m_hostname;
        }

        void QDnsRecordDataPTR::setHostname(const QString &hostname)
        {
            m_hostname = hostname;
        }
    }
}