//
// Created by robin on 1/23/18.
//

#include "dns-base/QDnsLabel.h"
#include "dns-base/RecordData/QDnsRecordDataSOA.h"

namespace QDns
{
    namespace Base
    {
        QDnsRecordDataSOA::QDnsRecordDataSOA()
                : QDnsRecordData()
        {
        }

        QDnsRecordDataSOA::QDnsRecordDataSOA(const QByteArray &packetData, quint32 pos)
                : QDnsRecordDataSOA()
        {
            QDataStream dataStream(packetData);
            dataStream.skipRawData(pos);
            parse(packetData, dataStream);
        }

        QDnsRecordDataSOA::QDnsRecordDataSOA(const QByteArray &packetData, QDataStream &dataStream)
                : QDnsRecordDataSOA()
        {
            parse(packetData, dataStream);
        }

        void QDnsRecordDataSOA::parse(const QByteArray &packetData, QDataStream &dataStream)
        {
            if (dataStream.device()->bytesAvailable() < 22)
            {
                m_isValid = false;
            }
            else
            {
                m_primaryNs = QDnsLabel::parse(packetData, dataStream);
                if (m_primaryNs.isNull())
                {
                    m_isValid = false;
                }
                else
                {
                    m_adminMb = QDnsLabel::parse(packetData, dataStream);
                    if (m_adminMb.isNull())
                    {
                        m_isValid = false;
                    }
                    else
                    {
                        dataStream >> m_serialNumber;
                        dataStream >> m_refreshInterval;
                        dataStream >> m_retryInterval;
                        dataStream >> m_expirationLimit;
                        dataStream >> m_minimumTtl;
                    }
                }
            }
        }

        QByteArray QDnsRecordDataSOA::serialize() const
        {
            QByteArray byteArray;
            QDataStream dataStream(&byteArray, QIODevice::WriteOnly);
            serialize(dataStream);
            return byteArray;
        }

        void QDnsRecordDataSOA::serialize(QDataStream &dataStream) const
        {
            QDnsLabel::serialize(m_primaryNs, dataStream);
            QDnsLabel::serialize(m_adminMb, dataStream);
            dataStream << m_serialNumber;
            dataStream << m_refreshInterval;
            dataStream << m_retryInterval;
            dataStream << m_expirationLimit;
            dataStream << m_minimumTtl;
        }

        QDnsRecordData *QDnsRecordDataSOA::clone() const
        {
            auto data = new QDnsRecordDataSOA();
            data->m_isValid = m_isValid;
            data->m_primaryNs = m_primaryNs;
            data->m_adminMb = m_adminMb;
            data->m_serialNumber = m_serialNumber;
            data->m_refreshInterval = m_refreshInterval;
            data->m_retryInterval = m_retryInterval;
            data->m_expirationLimit = m_expirationLimit;
            data->m_minimumTtl = m_minimumTtl;
            return data;
        }

        const QString &QDnsRecordDataSOA::getPrimaryNs() const
        {
            return m_primaryNs;
        }

        void QDnsRecordDataSOA::setPrimaryNs(const QString &primaryNs)
        {
            m_primaryNs = primaryNs;
        }

        const QString &QDnsRecordDataSOA::getAdminMb() const
        {
            return m_adminMb;
        }

        void QDnsRecordDataSOA::setAdminMb(const QString &adminMb)
        {
            m_adminMb = adminMb;
        }

        quint32 QDnsRecordDataSOA::getSerialNumber() const
        {
            return m_serialNumber;
        }

        void QDnsRecordDataSOA::setSerialNumber(quint32 serialNumber)
        {
            m_serialNumber = serialNumber;
        }

        quint32 QDnsRecordDataSOA::getRefreshInterval() const
        {
            return m_refreshInterval;
        }

        void QDnsRecordDataSOA::setRefreshInterval(quint32 refreshInterval)
        {
            m_refreshInterval = refreshInterval;
        }

        quint32 QDnsRecordDataSOA::getRetryInterval() const
        {
            return m_retryInterval;
        }

        void QDnsRecordDataSOA::setRetryInterval(quint32 retryInterval)
        {
            m_retryInterval = retryInterval;
        }

        quint32 QDnsRecordDataSOA::getExpirationLimit() const
        {
            return m_expirationLimit;
        }

        void QDnsRecordDataSOA::setExpirationLimit(quint32 expirationLimit)
        {
            m_expirationLimit = expirationLimit;
        }

        quint32 QDnsRecordDataSOA::getMinimumTtl() const
        {
            return m_minimumTtl;
        }

        void QDnsRecordDataSOA::setMinimumTtl(quint32 minimumTtl)
        {
            m_minimumTtl = minimumTtl;
        }
    }
}