//
// Created by robin on 1/9/18.
//

#include <gtest/gtest.h>
#include <QByteArray>
#include <QString>

#include "dns-base/QDnsHeader.h"

struct DnsHeaderTestParams
{
    bool isValid;
    const char* hexData;
    quint16 id;

    quint16 flags;
    bool flagQuery;
    QDns::Base::HeaderOpcode flagOpcode;
    bool flagAuthoritative;
    bool flagTruncated;
    bool flagRecursionDesired;
    bool flagRecursionAvailable;
    bool flagReserved0;
    bool flagAnswerAuthenticated;
    bool flagNonAuthenticatedAcceptable;
    QDns::Base::HeaderReplyCode flagReplyCode;

    quint16 queryCount;
    quint16 answerCount;
    quint16 authorityCount;
    quint16 additionalCount;
};

class DnsHeaderTest : public ::testing::TestWithParam<DnsHeaderTestParams>
{
};

TEST_P(DnsHeaderTest, parseAndGetters)
{
    auto params = GetParam();
    auto data = QByteArray::fromHex(params.hexData);
    auto header = QDns::Base::QDnsHeader(data);
    ASSERT_EQ(header.isValid(), params.isValid);

    if (header.isValid())
    {
        ASSERT_EQ(header.getId(), params.id);

        ASSERT_EQ(header.getFlags(), params.flags);
        ASSERT_EQ(header.isQuery(), params.flagQuery);
        ASSERT_EQ(header.getOpcode(), params.flagOpcode);
        ASSERT_EQ(header.isAuthoritative(), params.flagAuthoritative);
        ASSERT_EQ(header.isTruncated(), params.flagTruncated);
        ASSERT_EQ(header.isRecursionDesired(), params.flagRecursionDesired);
        ASSERT_EQ(header.isRecursionAvailable(), params.flagRecursionAvailable);
        ASSERT_EQ(header.isAnswerAuthenticated(), params.flagAnswerAuthenticated);
        ASSERT_EQ(header.isNonAuthenticatedAcceptable(), params.flagNonAuthenticatedAcceptable);
        ASSERT_EQ(header.getReplyCode(), params.flagReplyCode);

        ASSERT_EQ(header.getQueryCount(), params.queryCount);
        ASSERT_EQ(header.getAnswerCount(), params.answerCount);
        ASSERT_EQ(header.getAuthorityCount(), params.authorityCount);
        ASSERT_EQ(header.getAdditionalCount(), params.additionalCount);

        ASSERT_EQ(header.serialize().toHex(), QString(params.hexData).remove(" "));
    }
}

TEST_P(DnsHeaderTest, settersAndGetters1)
{
    auto params = GetParam();

    if (!params.isValid)
    {
        return;
    }

    auto data = QByteArray::fromHex(params.hexData);
    QDns::Base::QDnsHeader header;

    header.setId(params.id);
    header.setFlags(params.flags);
    header.setQueryCount(params.queryCount);
    header.setAnswerCount(params.answerCount);
    header.setAuthorityCount(params.authorityCount);
    header.setAdditionalCount(params.additionalCount);

    ASSERT_EQ(header.getId(), params.id);

    ASSERT_EQ(header.getFlags(), params.flags);
    ASSERT_EQ(header.isQuery(), params.flagQuery);
    ASSERT_EQ(header.getOpcode(), params.flagOpcode);
    ASSERT_EQ(header.isAuthoritative(), params.flagAuthoritative);
    ASSERT_EQ(header.isTruncated(), params.flagTruncated);
    ASSERT_EQ(header.isRecursionDesired(), params.flagRecursionDesired);
    ASSERT_EQ(header.isRecursionAvailable(), params.flagRecursionAvailable);
    ASSERT_EQ(header.isAnswerAuthenticated(), params.flagAnswerAuthenticated);
    ASSERT_EQ(header.isNonAuthenticatedAcceptable(), params.flagNonAuthenticatedAcceptable);
    ASSERT_EQ(header.getReplyCode(), params.flagReplyCode);

    ASSERT_EQ(header.getQueryCount(), params.queryCount);
    ASSERT_EQ(header.getAnswerCount(), params.answerCount);
    ASSERT_EQ(header.getAuthorityCount(), params.authorityCount);
    ASSERT_EQ(header.getAdditionalCount(), params.additionalCount);

    ASSERT_EQ(header.serialize().toHex(), QString(params.hexData).remove(" "));
}

TEST_P(DnsHeaderTest, settersAndGetters2)
{
    auto params = GetParam();

    if (!params.isValid)
    {
        return;
    }

    auto data = QByteArray::fromHex(params.hexData);
    QDns::Base::QDnsHeader header;

    header.setId(params.id);
    header.setIsQuery(params.flagQuery);
    header.setOpcode(params.flagOpcode);
    header.setIsAuthoritative(params.flagAuthoritative);
    header.setIsTruncated(params.flagTruncated);
    header.setIsRecursionDesired(params.flagRecursionDesired);
    header.setIsRecursionAvailable(params.flagRecursionAvailable);
    header.setFlagBits(9, (quint16) params.flagReserved0, 1);
    header.setIsAnswerAuthenticated(params.flagAnswerAuthenticated);
    header.setNonAuthenticatedAcceptable(params.flagNonAuthenticatedAcceptable);
    header.setReplyCode(params.flagReplyCode);
    header.setQueryCount(params.queryCount);
    header.setAnswerCount(params.answerCount);
    header.setAuthorityCount(params.authorityCount);
    header.setAdditionalCount(params.additionalCount);

    ASSERT_EQ(header.getId(), params.id);

    ASSERT_EQ(header.getFlags(), params.flags);
    ASSERT_EQ(header.isQuery(), params.flagQuery);
    ASSERT_EQ(header.getOpcode(), params.flagOpcode);
    ASSERT_EQ(header.isAuthoritative(), params.flagAuthoritative);
    ASSERT_EQ(header.isTruncated(), params.flagTruncated);
    ASSERT_EQ(header.isRecursionDesired(), params.flagRecursionDesired);
    ASSERT_EQ(header.isRecursionAvailable(), params.flagRecursionAvailable);
    ASSERT_EQ(header.isAnswerAuthenticated(), params.flagAnswerAuthenticated);
    ASSERT_EQ(header.isNonAuthenticatedAcceptable(), params.flagNonAuthenticatedAcceptable);
    ASSERT_EQ(header.getReplyCode(), params.flagReplyCode);

    ASSERT_EQ(header.getQueryCount(), params.queryCount);
    ASSERT_EQ(header.getAnswerCount(), params.answerCount);
    ASSERT_EQ(header.getAuthorityCount(), params.authorityCount);
    ASSERT_EQ(header.getAdditionalCount(), params.additionalCount);

    ASSERT_EQ(header.serialize().toHex(), QString(params.hexData).remove(" "));
}

INSTANTIATE_TEST_CASE_P(DnsHeaderTestInst,
                        DnsHeaderTest,
                        ::testing::Values(
                                DnsHeaderTestParams {false, ""},
                                DnsHeaderTestParams {false, "00"},
                                DnsHeaderTestParams {true,
                                                     "3c dc 01 00 00 01 00 00  00 00 00 00",
                                                     0x3cdc,

                                                     0x0100,
                                                     true,
                                                     QDns::Base::HeaderOpcode::Query,
                                                     false,
                                                     false,
                                                     true,
                                                     false,
                                                     false,
                                                     false,
                                                     false,
                                                     QDns::Base::HeaderReplyCode::NoError,

                                                     1,
                                                     0,
                                                     0,
                                                     0
                                },
                                DnsHeaderTestParams {true,
                                                     "3c dc 81 80 00 01 00 01  00 00 00 00",
                                                     0x3cdc,
                                                     0x8180,
                                                     false,
                                                     QDns::Base::HeaderOpcode::Query,
                                                     false,
                                                     false,
                                                     true,
                                                     true,
                                                     false,
                                                     false,
                                                     false,
                                                     QDns::Base::HeaderReplyCode::NoError,

                                                     1,
                                                     1,
                                                     0,
                                                     0
                                },

                                DnsHeaderTestParams {true,
                                                     "3c dc aa aa aa aa 55 55  55 ff ff 55",
                                                     0x3cdc,
                                                     0xaaaa,
                                                     false,
                                                     QDns::Base::HeaderOpcode::Update,
                                                     false,
                                                     true,
                                                     false,
                                                     true,
                                                     false,
                                                     true,
                                                     false,
                                                     QDns::Base::HeaderReplyCode::NotZone,

                                                     43690,
                                                     21845,
                                                     22015,
                                                     65365
                                },
                                DnsHeaderTestParams {true,
                                                     "3c dc 55 55 55 ff aa aa  ff 55 55 55",
                                                     0x3cdc,
                                                     0x5555,
                                                     true,
                                                     (QDns::Base::HeaderOpcode) 10,
                                                     true,
                                                     false,
                                                     true,
                                                     false,
                                                     true,
                                                     false,
                                                     true,
                                                     QDns::Base::HeaderReplyCode::Refused,

                                                     22015,
                                                     43690,
                                                     65365,
                                                     21845
                                }
                        ));

TEST(DnsHeaderTest, getFlagBits)
{
    QDns::Base::QDnsHeader header;

    header.setFlags(0x5555);
    ASSERT_EQ(header.getFlagBits(0, 1), 0);
    ASSERT_EQ(header.getFlagBits(3, 4), 5);
}

TEST(DnsHeaderTest, getFlagBitIndex)
{
    ASSERT_EQ(QDns::Base::QDnsHeader::getFlagBitIndex(0), 15);
    ASSERT_EQ(QDns::Base::QDnsHeader::getFlagBitIndex(1), 14);
    ASSERT_EQ(QDns::Base::QDnsHeader::getFlagBitIndex(14), 1);
    ASSERT_EQ(QDns::Base::QDnsHeader::getFlagBitIndex(15), 0);
}