//
// Created by robin on 1/14/18.
//

#include <gtest/gtest.h>
#include <QByteArray>
#include <QString>
#include <QtNetwork/QHostAddress>
#include <dns-base/RecordData/QDnsRecordDataPTR.h>

struct DnsRDataPTRTestParams
{
    bool isValid;
    const char* hexData;
    quint32 pos;

    QString hostname;
};

class DnsRDataPTRTest : public ::testing::TestWithParam<DnsRDataPTRTestParams>
{
};

TEST_P(DnsRDataPTRTest, parseAndGetters)
{
    auto params = GetParam();
    auto data = QByteArray::fromHex(params.hexData);
    auto rdata = dynamic_cast<QDns::Base::QDnsRecordDataPTR*>(QDns::Base::QDnsRecordDataPTR(data, params.pos).clone());
    ASSERT_NE(rdata, nullptr);
    ASSERT_EQ(rdata->isValid(), params.isValid);

    if (rdata->isValid())
    {
        ASSERT_EQ(params.hostname, rdata->getHostname());

        ASSERT_EQ(rdata->serialize().toHex(), QString(params.hexData).remove(" "));
    }
    delete rdata;
}

INSTANTIATE_TEST_CASE_P(DnsRDataPTRTestInst,
                        DnsRDataPTRTest,
                        ::testing::Values(
                                DnsRDataPTRTestParams {false, "", 0},
                                DnsRDataPTRTestParams {true, "00", 0, ""},
                                DnsRDataPTRTestParams {true, "03777777067274686f6e6903636f6d00", 0, "www.rthoni.com"}
                        ));