//
// Created by robin on 2/7/18.
//

#include "dns-client/QDnsTcpClient.h"

namespace QDns
{
    namespace Client
    {
        QDnsTcpClient::QDnsTcpClient(const QHostAddress &host, quint16 port)
            : QDnsClient(host, port)
            , m_socket(new QTcpSocket(this))
            , m_size(0)
        {
            connect(m_socket, &QTcpSocket::readyRead, this, &QDnsTcpClient::onSocketReadyRead);
            connect(m_socket, &QTcpSocket::stateChanged, this, &QDnsTcpClient::onSocketStateChanged);
            connect(m_socket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(onSocketError(QAbstractSocket::SocketError)));
        }

        void QDnsTcpClient::connectToHost()
        {
            m_socket->connectToHost(m_host, m_port);
        }

        void QDnsTcpClient::disconnectFromHost()
        {
            m_socket->disconnectFromHost();
        }

        void QDnsTcpClient::doQuery(QDns::Base::QDnsPacket packet)
        {
            auto answerRaw = packet.serialize(); // TODO split packet if too big (>65535)
            QByteArray size;
            QDataStream dataStream(&size, QIODevice::WriteOnly);
            dataStream << (quint16) answerRaw.size();
            m_socket->write(size.constData(), size.size());
            m_socket->write(answerRaw.constData(), answerRaw.size());
        }

        void QDnsTcpClient::onSocketReadyRead()
        {
            m_buffer.append(m_socket->readAll());

            bool readAgain;

            do
            {
                readAgain = false;
                if (m_buffer.size() > 2 && m_size == 0)
                {
                    QDataStream dataStream(m_buffer);
                    dataStream >> m_size;
                }
                if (m_size != 0 && m_buffer.size() >= m_size + 2)
                {
                    auto data = m_buffer.mid(2, m_size);
                    m_buffer.remove(0, m_size + 2);
                    m_size = 0;
                    readAgain = m_buffer.size() != 0;
                    processData(ProcessDataParams
                                        {
                                                data,
                                                m_socket->peerAddress(),
                                                m_socket->peerPort()
                                        });
                }
            } while (readAgain);
        }

        void QDnsTcpClient::onSocketStateChanged(QAbstractSocket::SocketState socketState)
        {
            if (socketState == QAbstractSocket::SocketState::HostLookupState)
            {
                setState(ClientState::Connecting);
            }
            else if (socketState == QAbstractSocket::SocketState::ConnectedState)
            {
                setState(ClientState::Connected);
            }
            else if (socketState == QAbstractSocket::SocketState::ClosingState)
            {
                setState(ClientState::Disconnecting);
            }
            else if (socketState == QAbstractSocket::SocketState::UnconnectedState)
            {
                setState(ClientState::Disconnected);
            }
        }

        void QDnsTcpClient::onSocketError(QAbstractSocket::SocketError error)
        {
            emit onError(OnErrorParams
                                 {
                                         0,
                                         SocketError,
                                         error,
                                         QDns::Base::QDnsPacket(""),
                                         QByteArray(),
                                         QDns::Base::QDnsPacket(""),
                                         m_host,
                                         m_port
                                 });
        }
    }
}