//
// Created by robin on 2/2/18.
//

#include "dns-client/QDnsUdpClient.h"

namespace QDns
{
    namespace Client
    {
        QDnsUdpClient::QDnsUdpClient(const QHostAddress &host, quint16 port)
            : QDnsClient(host, port)
            , m_socket(new QUdpSocket(this))
        {
            connect(m_socket, &QUdpSocket::readyRead, this, &QDnsUdpClient::onReadyRead);
            connect(m_socket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(onSocketError(QAbstractSocket::SocketError)));
        }

        void QDnsUdpClient::connectToHost()
        {
            setState(ClientState::Connecting);
            setState(ClientState::Connected);
        }

        void QDnsUdpClient::disconnectFromHost()
        {
            setState(ClientState::Disconnecting);
            setState(ClientState::Disconnected);
        }

        void QDnsUdpClient::doQuery(QDns::Base::QDnsPacket packet)
        {
            m_socket->writeDatagram(packet.serialize(), m_host, m_port);
        }

        void QDnsUdpClient::onReadyRead()
        {
            while (m_socket->hasPendingDatagrams())
            {
                QByteArray datagram;
                datagram.resize((int) m_socket->pendingDatagramSize());
                QHostAddress senderAddress;
                quint16 senderPort;

                m_socket->readDatagram(datagram.data(), datagram.size(), &senderAddress, &senderPort); // TODO check sender and port? May not answer from same ip?
                processData(ProcessDataParams
                                    {
                                            datagram,
                                            senderAddress,
                                            senderPort
                                    });
            }
        }

        void QDnsUdpClient::onSocketError(QAbstractSocket::SocketError error)
        {
            emit onError(OnErrorParams
                                 {
                                         0,
                                         SocketError,
                                         error,
                                         QDns::Base::QDnsPacket(""),
                                         QByteArray(),
                                         QDns::Base::QDnsPacket(""),
                                         m_host,
                                         m_port
                                 });
        }
    }
}